@@ -19,9 +19,16 @@ options(
   testthat.progress.max_fails = 50
 )
 
+# rlang::global_entrace()
+
 library(rlang, warn.conflicts=FALSE)
 
-options(trialmaster_pw="0")
+
+edc_options(
+  trialmaster_pw="0", 
+  edc_lookup_overwrite_warn=FALSE
+)
+
 # getOption("trialmaster_pw")
 
 # cachename="trialmaster_export_2022-08-25 15h16.rds"
@@ -56,7 +63,6 @@ clean_cache = function(){
 }
 v=utils::View
 
-
 snapshot_review_bg = function(...){
   # brw = function(url) .Call("rs_browseURL", url, PACKAGE="(embedding)")
   brw = Sys.getenv("R_BROWSER")
@@ -122,5 +128,36 @@ expect_classed_conditions = function(expr, message_class=NULL, warning_class=NUL
   x
 }
 
+
+condition_overview = function(expr){
+  tryCatch2(expr) %>% attr("overview")
+}
+tryCatch2 = function(expr){
+  errors = list()
+  warnings = list()
+  messages = list()
+  rtn = withCallingHandlers(tryCatch(expr, error = function(e) {
+    errors <<- c(errors, list(e))
+    return("error")
+  }), warning = function(w) {
+    warnings <<- c(warnings, list(w))
+    invokeRestart("muffleWarning")
+  }, message = function(m) {
+    messages <<- c(messages, list(m))
+    invokeRestart("muffleMessage")
+  })
+  attr(rtn, "errors") = unique(map_chr(errors, conditionMessage))
+  attr(rtn, "warnings") = unique(map_chr(warnings, conditionMessage))
+  attr(rtn, "messages") = unique(map_chr(messages, conditionMessage))
+  x = c(errors, warnings, messages) %>% unique()
+  attr(rtn, "overview") = tibble(type = map_chr(x, ~ifelse(inherits(.x, 
+                                                                    "error"), "Error", ifelse(inherits(.x, "warning"), "Warning", 
+                                                                                              "Message"))), class = map_chr(x, ~class(.x) %>% glue_collapse("/")), 
+                                 message = map_chr(x, ~conditionMessage(.x)))
+  rtn
+}
+
+
 clean_cache()
-message("Helper-init loaded")
+cli::cli_inform(c(v="Initializer {.file helper-init_dataset.R} loaded: 
+                     is_testing={is_testing()}, is_parallel={is_parallel()}"))
