@@ -56,13 +56,15 @@ test_that("get_folder_datetime() works", {
 
 test_that("get_lookup() works", {
   
-  lookup = list(i=crosstable::iris2, m=mtcars) %>% get_lookup()
-  expect_equal(lengths(lookup$names), c(m=11, i=5))
+  x = edc_example()
+  x$.lookup=NULL
+  lookup = get_lookup(x)
+  expect_equal(lengths(lookup$names), c(db0=5,db2=5,db3=6,db1=6))
   expect_true(all(nzchar(lookup$labels$i)))
   expect_false(any(nzchar(lookup$labels$m)))
   # lookup %>% unnest(everything())
   
-  x = list(i=crosstable::iris2, mtcars)
+  x = list(i=iris, mtcars)
   get_lookup(x) %>% 
     expect_error(class="edc_lookup_unnamed")
   x = list(date_extraction=1, datetime_extraction=1, .lookup=mtcars)
@@ -71,13 +73,14 @@ test_that("get_lookup() works", {
 })
 
 test_that("find_keyword() works", {
-  lookup = list(iris2=crosstable::iris2, mtcars2=crosstable::mtcars2) %>% get_lookup()
-  x1=find_keyword("hp", data=lookup)
-  expect_equal(x1$names, c("hp", "hp_date"))
-  x2=find_keyword("number|date", data=lookup)
-  expect_equal(x2$names, c("cyl", "gear", "carb", "hp_date", "qsec_posix"))
-  x3=find_keyword("number|date", data=lookup, ignore_case=FALSE)
-  expect_equal(x3$names, "hp_date")
+  x = edc_example()
+  # x$.lookup %>% unnest() %>% v
+  x1=find_keyword("visit", data=x$.lookup)
+  expect_setequal(x1$names, paste0("date", 1:10))
+  x2=find_keyword("id|\\(", data=x$.lookup)
+  expect_equal(unique(x2$names), c("SUBJID", "age"))
+  x3=find_keyword("id|\\(", data=x$.lookup, ignore_case=FALSE)
+  expect_equal(unique(x3$names), "age")
 })
 
 
@@ -135,6 +138,13 @@ test_that("7zip not in the path", {
 
 
 
+# Options -------------------------------------------------------------------------------------
+
+
+test_that("No missing options", {
+  missing_options = missing_options_helper()
+  expect_identical(missing_options, character(0))
+})
 
 # Expect --------------------------------------------------------------------------------------
 
